@@ -36,7 +36,7 @@ use Dancer::Plugin::Database;
 use HTML::Table::FromDatabase;
 use CGI::FormBuilder;
 
-our $VERSION = '0.70';
+our $VERSION = '0.71';
 
 =encoding utf8
 
@@ -141,6 +141,17 @@ The prefix for the routes which will be created.  Given a prefix of C</widgets>,
 then you can go to C</widgets/new> to create a new Widget, and C</widgets/42> to
 edit the widget with the ID (see keu_column) 42.
 
+Don't confuse this with Dancer's C<prefix> setting, which would be prepended
+before the prefix you pass to this plugin.  For example, if you used:
+
+    prefix '/foo';
+    simple_crud(
+        prefix => 'bar',
+        ...
+    );
+
+... then you'd end up with e.g. C</foo/bar> as the record listing page.
+
 =item C<db_table> (required)
 
 The name of the database table.
@@ -321,6 +332,14 @@ sub simple_crud {
         $args{prefix} = '/' . $args{prefix};
     }
 
+    # If there's a Dancer prefix in use, as well as a prefix we're told about,
+    # then _construct_url() will need to be told about that later so it can
+    # construct URLs.  It can't just call Dancer::App->current->prefix itself,
+    # though, as the prefix may have changed by the time the code is actually
+    # running.  (See RT #73620.)   So, we need to grab it here and add it to
+    # %args, so it can see it later.
+    $args{dancer_prefix} = Dancer::App->current->prefix || '';
+
     if (!$args{db_table}) { die "Need table name!"; }
 
     # Accept deleteable as a synonym for deletable
@@ -383,7 +402,8 @@ CONFIRMDELETE
         # A route for POST requests, to actually delete the record
         post qr[$args{prefix}/delete/?(.+)?$] => sub {
             my ($id) = params->{record_id} || splat;
-            database->quick_delete($table_name, { $key_column => $id })
+            my $dbh = database($args{db_connection_name});
+            $dbh->quick_delete($table_name, { $key_column => $id })
                 or return _apply_template("<p>Failed to delete!</p>",
                 $args{'template'});
 
@@ -409,7 +429,7 @@ sub _create_add_edit_route {
     my $default_field_values;
     if ($id) {
         $default_field_values
-            = database->quick_select($table_name, { $key_column => $id });
+            = $dbh->quick_select($table_name, { $key_column => $id });
     }
 
     # Find out about table columns:
@@ -478,7 +498,7 @@ sub _create_add_edit_route {
             # Find out the possible values for this column from the other table:
             my %possible_values;
             debug "Looking for rows for foreign relation: " => $foreign_key;
-            for my $row (database->quick_select($foreign_key->{table}, {})) {
+            for my $row ($dbh->quick_select($foreign_key->{table}, {})) {
                 debug "Row from foreign relation: " => $row;
                 $possible_values{ $row->{ $foreign_key->{key_column} } }
                     = $row->{ $foreign_key->{label_column} };
@@ -505,6 +525,7 @@ sub _create_add_edit_route {
         validate => $validation,
         method   => 'post',
         action   => _construct_url(
+            $args->{dancer_prefix},
             $args->{prefix},
             (
                 params->{id}
@@ -577,11 +598,11 @@ sub _create_add_edit_route {
         if (exists params->{$key_column}) {
 
             # We're editing an existing record
-            $success = database->quick_update($table_name,
+            $success = $dbh->quick_update($table_name,
                 { $key_column => params->{$key_column} }, \%params);
             $verb = 'update';
         } else {
-            $success = database->quick_insert($table_name, \%params);
+            $success = $dbh->quick_insert($table_name, \%params);
             $verb = 'create new';
         }
 
@@ -589,7 +610,7 @@ sub _create_add_edit_route {
 
             # Redirect to the list page
             # TODO: pass a param to cause it to show a message?
-            redirect _construct_url($args->{prefix});
+            redirect _construct_url($args->{dancer_prefix}, $args->{prefix});
             return;
         } else {
 
@@ -674,18 +695,18 @@ SEARCHFORM
     if ($args->{foreign_keys}) {
         while (my ($col, $foreign_key) = each(%{ $args->{foreign_keys} })) {
             @select_cols = grep { $_ ne $col } @select_cols;
-            my $ftable = database->quote_identifier($foreign_key->{table});
+            my $ftable = $dbh->quote_identifier($foreign_key->{table});
             my $fcol
-                = database->quote_identifier($foreign_key->{label_column});
+                = $dbh->quote_identifier($foreign_key->{label_column});
             my $lcol
-                = database->quote_identifier($args->{labels}{$col} || $col);
+                = $dbh->quote_identifier($args->{labels}{$col} || $col);
             push @foreign_cols, "$ftable.$fcol AS $lcol";
         }
     }
 
     my $col_list = join(
         ',',
-        map({ $table_name . "." . database->quote_identifier($_) }
+        map({ $table_name . "." . $dbh->quote_identifier($_) }
             @select_cols),
         @foreign_cols,    # already assembled from quoted identifiers
     );
@@ -698,9 +719,9 @@ SEARCHFORM
     # If we have foreign key relationship info, we need to join on those tables:
     if ($args->{foreign_keys}) {
         while (my ($col, $foreign_key) = each %{ $args->{foreign_keys} }) {
-            my $ftable = database->quote_identifier($foreign_key->{table});
-            my $lkey   = database->quote_identifier($col);
-            my $rkey = database->quote_identifier($foreign_key->{key_column});
+            my $ftable = $dbh->quote_identifier($foreign_key->{table});
+            my $lkey   = $dbh->quote_identifier($col);
+            my $rkey   = $dbh->quote_identifier($foreign_key->{key_column});
 
             # Identifiers quoted above, and $table_name quoted further up, so
             # all safe to interpolate
@@ -713,8 +734,10 @@ SEARCHFORM
         my ($column_data)
             = grep { lc $_->{COLUMN_NAME} eq lc params->{searchfield} }
             @{$columns};
-        debug(    "Searching on $column_data->{COLUMN_NAME} which is a "
-                . "$column_data->{TYPE_NAME}");
+        debug(
+            "Searching on $column_data->{COLUMN_NAME} which is a "
+            . "$column_data->{TYPE_NAME}"
+        );
 
         if ($column_data) {
             my $search_value = params->{'q'};
@@ -733,7 +756,7 @@ SEARCHFORM
                 "<p>Showing results from searching for '%s' in '%s'",
                 params->{'q'}, params->{searchfield});
             $html .= sprintf '&mdash;<a href="%s">Reset search</a></p>',
-                _construct_url($args->{prefix});
+                _construct_url($args->{dancer_prefix}, $args->{prefix});
         }
     }
 
@@ -746,7 +769,7 @@ SEARCHFORM
 
         my @formats = qw/csv tabular json xml/;
 
-        my $url = _construct_url($args->{prefix})
+        my $url = _construct_url($args->{dancer_prefix}, $args->{prefix})
             . "?o=$o&d=$d&q=$q&searchfield=$sf&p=$page";
 
         $html
@@ -782,7 +805,7 @@ SEARCHFORM
                 $direction = $opposite_order_by_direction;
                 $direction_char = ($direction eq "asc") ? "&uarr;" : "&darr;";
             }
-            my $url = _construct_url($args->{prefix})
+            my $url = _construct_url($args->{dancer_prefix}, $args->{prefix})
                 . "?o=$col_name&d=$direction&q=$q&searchfield=$sf";
             $col_name =>
                 "<a href=\"$url\">$col_name&nbsp;$direction_char</a>";
@@ -790,7 +813,7 @@ SEARCHFORM
 
         $query
             .= " ORDER BY $table_name."
-            . database->quote_identifier($order_by_column) . " "
+            . $dbh->quote_identifier($order_by_column) . " "
             . $order_by_direction . " ";
     }
 
@@ -807,7 +830,7 @@ SEARCHFORM
         my $offset = $page_size * $page;
         my $limit  = $page_size;
 
-        my $url = _construct_url($args->{prefix})
+        my $url = _construct_url($args->{dancer_prefix}, $args->{prefix})
             . "?o=$o&d=$d&q=$q&searchfield=$sf";
         $html .= "<p>";
         if ($page > 0) {
@@ -838,7 +861,7 @@ SEARCHFORM
     my $sth = $dbh->prepare($query);
     $sth->execute()
         or die "Failed to query for records in $table_name - "
-        . database->errstr;
+        . $dbh->errstr;
 
     if ($args->{downloadable} && params->{format}) {
 
@@ -857,12 +880,17 @@ SEARCHFORM
                     my $action_links;
                     if ($args->{editable}) {
                         my $edit_url
-                            = _construct_url($args->{prefix}, "/edit/$id");
+                            = _construct_url(
+                                $args->{dancer_prefix}, $args->{prefix}, 
+                                "/edit/$id"
+                            );
                         $action_links
                             .= qq[<a href="$edit_url" class="edit_link">Edit</a>];
                         if ($args->{deletable}) {
-                            my $del_url = _construct_url($args->{prefix},
-                                "/delete/$id");
+                            my $del_url = _construct_url(
+                                $args->{dancer_prefix}, $args->{prefix},
+                                "/delete/$id"
+                            );
                             $action_links
                                 .= qq[ / <a href="$del_url" class="delete_link"]
                                 . qq[ onclick="delrec('$id'); return false;">]
@@ -882,12 +910,15 @@ SEARCHFORM
 
     if ($args->{editable}) {
         $html .= sprintf '<a href="%s">Add a new %s</a></p>',
-            _construct_url($args->{prefix}, '/add'), $args->{record_title};
+            _construct_url($args->{dancer_prefix}, $args->{prefix}, '/add'), 
+            $args->{record_title};
 
         # Append a little Javascript which asks for confirmation that they'd
         # like to delete the record, then makes a POST request via a hidden
         # form.  This could be made AJAXy in future.
-        my $del_action = _construct_url($args->{prefix}, '/delete');
+        my $del_action = _construct_url(
+            $args->{dancer_prefix}, $args->{prefix}, '/delete'
+        );
         $html .= <<DELETEJS;
 <form name="deleteform" method="post" action="$del_action">
 <input name="record_id" type="hidden">
@@ -1022,8 +1053,6 @@ sub _find_columns {
 # would return: /foo/bar/baz
 sub _construct_url {
     my @url_parts = @_;
-    my $prefix_setting = Dancer::App->current->prefix || '';
-    unshift @url_parts, $prefix_setting;
 
     # Just concatenate all parts together, then deal with multiple slashes.
     # This could be problematic if any URL was ever supposed to contain multiple
